#pragma once

/*
 * SPDX-FileCopyrightText: 2003-2009 Craig Drummond <craig@kde.org>
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#define FONTLIST_TAG "fontlist"
#define FONT_TAG "font"
#define FILE_TAG "file"
#define PATH_ATTR "path"
#define NAME_ATTR "name"
#define FOUNDRY_ATTR "foundry"
#define FAMILY_ATTR "family"
#define FAMILY_TAG "family"
#define WEIGHT_ATTR "weight"
#define WIDTH_ATTR "width"
#define SLANT_ATTR "slant"
#define SCALABLE_ATTR "scalable"
#define FACE_ATTR "face"
#define LANGS_ATTR "langs"
#define SYSTEM_ATTR "system"
#define ERROR_ATTR "error"
#define LANG_SEP ","
